//===========================================================================
//
//  CRYSTALFONTZ 
//
//  This code uses the 8-bit parallel MCU mode of the display.
//
//    https://www.crystalfontz.com/product/cfag160160d5tfhvgb
//
//  The controller is a RAiO Technology RA8803/8822:
//    https://www.crystalfontz.com/controllers/RAiOTechnology/RA8803-8822/
//
//  Seeeduino v4.2, an open-source 5v/3.3v capable Arduino clone.
//    https://www.seeedstudio.com/Seeeduino-V4.2-p-2517.html
//    https://github.com/SeeedDocument/SeeeduinoV4/raw/master/resources/Seeeduino_v4.2_sch.pdf
//===========================================================================
//
//
//
//===========================================================================
//This is free and unencumbered software released into the public domain.
//
//Anyone is free to copy, modify, publish, use, compile, sell, or
//distribute this software, either in source code form or as a compiled
//binary, for any purpose, commercial or non-commercial, and by any
//means.
//
//In jurisdictions that recognize copyright laws, the author or authors
//of this software dedicate any and all copyright interest in the
//software to the public domain. We make this dedication for the benefit
//of the public at large and to the detriment of our heirs and
//successors. We intend this dedication to be an overt act of
//relinquishment in perpetuity of all present and future rights to this
//software under copyright law.
//
//THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
//MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
//IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
//OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
//ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//OTHER DEALINGS IN THE SOFTWARE.
//
//For more information, please refer to <http://unlicense.org/>
//============================================================================
//
//
//
//============================================================================
// LCD & USD control lines
//   ARD      | Port |			  |  Function					| Wire
//------------+------+------------+-----------------------------+------------
//  GND	      |		 | #1 #2      |  GROUND						| Black
//  5V	      |		 | #3         |  POWER 5V					| Red
// -----------+------+------------+-----------------------------+------------
//  #8/D8     |  PB0 | #7         |  Chip Enable Signal (CS)	| Purple
//  #9/D9     |  PB1 | #8         |  Data/Instruction   (RS)	| Yellow
// #10/D10    |  PB2 | #5         |  Read/Write         (WR)	| Grey
// #10/D11    |  PB3 | #6         |  Read/Write         (RD)	| Brown
// #11/D12    |  PB4 | #10        |  Reset		        (RESET) | Blue
// #11/D13    |  PB5 | #9         |  Busy		        (BUSY)  | White
// -----------+------+------------+-----------------------------+------------
//  #0/D0     |  PD0 | #11        |  LCD_D10 (DB0)				| Purple
//  #1/D1     |  PD1 | #12        |  LCD_D11 (DB1)				| Green
//  #2/D2     |  PD2 | #13        |  LCD_D12 (DB2)				| Orange
//  #3/D3     |  PD3 | #14        |  LCD_D13 (DB3)				| Yellow
//  #4/D4     |  PD4 | #15        |  LCD_D14 (DB4)				| Grey
//  #5/D5     |  PD5 | #16        |  LCD_D15 (DB5)				| Brown
//  #6/D6     |  PD6 | #17        |  LCD_D16 (DB6)				| White
//  #7/D7     |  PD7 | #18        |  LCD_D17 (DB7)				| Black
// -----------+------+------------+-----------------------------+------------
//  Note: there also needs to be a potentiometer connected from pin4 (Vo) to pin 20(Vee)
//         the adjustable resitance needs to be hooked up to Vo and the remaining pin
//         on the POT should be connected to GND
//============================================================================

#include <stdio.h>
#include "command.c"
#include <SoftwareSerial.h>

SoftwareSerial mySerial(14, 15);

void setup()
{
	//Set the ports to begin states
	SET_CS;
	SET_WR;
	SET_RD;
	SET_RESET;
	//Set port input/output
	DDRD = 0xff;
	DDRB = 0x1f;


	//Blank out the screen for debugging
	mySerial.begin(9600);
	for (int i = 0; i < 100; i++)
	{
		mySerial.println();
	}


    //Tiny wait for power to stabilize
	delay(50);
	CLR_RESET; // LCD_RST = 0; // LCD RESET-pin active low.
	delay(250); // Reset 250ms: 8-2 Power On/Reset Process
	SET_RESET; // LCD_RST = 1; // LCD RESET-pin Normal high.
	delay(50); // Reset 250ms: 8-2 Power On/Reset Process

	LCD_Initial();

	mySerial.println("starting loop");
}



#define pictureDemo	1
#define linesDemo	1
#define printLines	1
#define letterDemo	1
void loop()
{
	mySerial.println("Top of loop");

#if (linesDemo == 1)
	LCD_Fill(0xFF);  
	delay(1000);
	LCD_Fill(0xF0);
	delay(1000);
	LCD_Fill(0x00);
	delay(1000);
	LCD_Fill(0x0F);
	delay(1000);
	LCD_Clear();
#endif

#if (pictureDemo == 1 )


	GotoXY(0, 0); // Setup Cursor Position

	for (int i = 0; i < 3200; i++)
	{
			LCD_DataWrite(pgm_read_byte(&Dexter_1BPP[i]));

	}
	delay(2000);
	
	GotoXY(0, 0); // Setup Cursor Position
	for (int i = 0; i < 3200; i++)
	{
		LCD_DataWrite(pgm_read_byte(&Starbucks_1BPP[i]));
	}
	delay(2000);

#endif // pictureDemo

	LCD_Clear();
	GotoXY(0, 0); // Setup Cursor Position

	mySerial.print("0x00:");
	mySerial.println(LCD_CmdRead(0x01));

	mySerial.print("0x10:");
	mySerial.println(LCD_CmdRead(0x10));
	delay(1000);

#if (printLines == 1)
	LCD_CmdWrite(0x00, 0xCD); // LCD Control Register(WLCR)

	GotoXY(0, 0);
	PrintStr("Line 1", 0);
	GotoXY(7, 1 * 16);
	PrintStr("Line 2", 0);
	GotoXY(14, 2 * 16);
	PrintStr("Line 3", 0);
	GotoXY(7, 3 * 16);
	PrintStr("Line 4", 0);
	GotoXY(0, 4 * 16);
	PrintStr("Line 5", 0);
	GotoXY(7, 5 * 16);
	PrintStr("Line 6", 0);
	GotoXY(14, 6 * 16);
	PrintStr("Line 7", 0);
	GotoXY(7, 7 * 16);
	PrintStr("Line 8", 0);
	GotoXY(0, 8 * 16);
	PrintStr("Line 9", 0);
	GotoXY(7, 9 * 16);
	PrintStr("Line 10", 0);
	LCD_CmdWrite(0x00, 0xC5);
	delay(5000);
#endif

#if (letterDemo ==1)
	LCD_CmdWrite(0x00, 0xCD);
	GotoXY(0, 0);
		CLR_CS;
		SET_RD; 
		SET_RS; // RS = 1;
	for(int i = 0; i < 5000; i++)
	{
		PORTD = i;
		CLR_WR; // WR = 0;
		delay(0);
		SET_WR; // WR = 1;
	}
		SET_RS; // RS = 1;
		SET_CS; //RA8803/8822 Chip Disable.		
	delay(4000);
	LCD_CmdWrite(0x00, 0xC5); // LCD Control Register(WLCR)
#endif

}